# SQLAlchemy

**FastAPI Users** provides the necessary tools to work with SQL databases thanks to [SQLAlchemy ORM with asyncio](https://docs.sqlalchemy.org/en/14/orm/extensions/asyncio.html).

!!! warning
    The previous adapter using `encode/databases` is now deprecated but can still be installed using `fastapi-users[sqlalchemy]`.

## Asynchronous driver

To work with your DBMS, you'll need to install the corresponding asyncio driver. The common choices are:

* For PostgreSQL: `pip install asyncpg`
* For SQLite: `pip install aiosqlite`

For the sake of this tutorial from now on, we'll use a simple SQLite databse.

## Setup User table

Let's declare our SQLAlchemy `User` table.

```py hl_lines="15 16"
--8<-- "docs/src/db_sqlalchemy.py"
```

As you can see, **FastAPI Users** provides a mixin that will include base fields for our `User` table. You can of course add you own fields there to fit to your needs!

## Implement a function to create the tables

We'll now create an utility function to create all the defined tables.

```py hl_lines="23-25"
--8<-- "docs/src/db_sqlalchemy.py"
```

This function can be called, for example, during the initialization of your FastAPI app.

!!! warning
    In production, it's strongly recommended to setup a migration system to update your SQL schemas. See [Alembic](https://alembic.sqlalchemy.org/en/latest/).

## Create the database adapter dependency

The database adapter of **FastAPI Users** makes the link between your database configuration and the users logic. It should be generated by a FastAPI dependency.

```py hl_lines="28-34"
--8<-- "docs/src/db_sqlalchemy.py"
```

Notice that we define first a `get_async_session` dependency returning us a fresh SQLAlchemy session to interact with the database.

It's then used inside the `get_user_db` dependency to generate our adapter. Notice that we pass it three things:

* A reference to your [`UserDB` model](../models.md).
* The `session` instance we just injected.
* The `UserTable` variable, which is the actual SQLAlchemy model.
